@@ -1,6 +1,6 @@
 =head1 NAME
 
-Log::Handler - A simple handler to log messages to log files.
+Log::Handler - A handler to log messages to different outputs.
 
 =head1 SYNOPSIS
 
@@ -8,34 +8,43 @@ Log::Handler - A simple handler to log messages to log files.
 
     my $log = Log::Handler->new();
 
-    $log->add(filename => \*STDOUT);
+    $log->add(File => {
+        filename => 'file.log',
+        maxlevel => 'debug',
+        minlevel => 'warn',
+        newline  => 1,
+    });
 
     $log->alert("foo bar");
 
 =head1 DESCRIPTION
 
-This module is just a simple object oriented log file handler and very easy to use.
+This module is just a simple object oriented log handler and very easy to use.
 It's possible to define a log level for your programs and control the amount of
-informations that will be logged to log files. In addition it's possible to configure
-how you wish to open the log files - transient or permanent - and lock and unlock the
-log files by each write operation. If you wish you can assign the handler to check the
-inode of the log files (not on Windows). That could be very useful if a rotate
-mechanism moves and zip the log file.
+informations that will be logged to one or more outputs. The outputs are different
+modules in the Log::Handler::Logger subtree and it's possible to extend it.
 
 =head1 WHAT IS NEW, WHAT IS DEPRECATED
 
-=head2 More than one log file
+=head2 Backward compatibilities
 
-Since version 0.39 the method C<add()> is totaly new. With this method it's now possible to
-add further log files to the handler and define a own level range each log file. As example
-you can log the levels 0-4 (emerg-warn) to log file A and the levels 4-7 (warn-debug)
-to log file B.
+As I re-designed the Log::Handler it was my wish to support the old style in version 0.38.
+The one exception is that the option C<redirect_to_stderr> doesn't exist any more. In all
+other cases you can use all things from 0.38.
+
+=head2 More than one output
+
+Since version 0.38_01 the method C<add()> is totaly new. With this method it's now possible
+to add output objects as much as you wish, each with an own level range and different other
+options. As example you can log the levels 0-4 (emerg-warn) to one object and the levels
+4-7 (warn-debug) to another object. In addition you want to forward all log levels to a
+subroutine. That's possible.
 
 =head2 Log::Handler::Logger
 
-The main logic of Log::Handler is moved to Log::Handler::Logger. If you add a new log file
-to the handler the file got it's own Log::Handler::Logger object and the Log::Handler just
-dispatch messages to the file objects.
+The main logic of Log::Handler is moved to Log::Handler::Logger. The Log::Handler just
+manage all logger objects and forward messages to it if the current log handler would
+log.
 
 =head2 Placeholder
 
@@ -43,18 +52,28 @@ Placeholders are now available for the prefix in printf style. The old style of
 is deprecated, instead you have to use %L as example. In addition a postfix option is
 available now. Take a look to the documentation of prefix and postfix.
 
+=head2 Configuration file
+
+Now it's possible to load the configuration from a file or define defaults. For this purpose
+I wrote C<Log::Handler::Config>.
+
 =head2 Kicked methods
 
 The methods C<close()>, C<get_prefix()> and C<set_prefix()> are not available any more.
 
+=head2 Kicked options
+
+C<rewrite_to_stderr>.
+
 =head2 trace()
 
-The method C<trace()> writes C<caller()> informations to all log files by default.
+The method C<trace()> writes C<caller()> informations to all logger by default.
 It's possible to disable this by set trace to 0.
 
-=head2 Examples
+=head2 Further releases
 
-Take a look to the example directory of the distribution if you need a code example.
+Extensions and further changes are planed. I hope I have enough time to implement my
+ideas as soon as possible!
 
 =head1 METHODS
 
@@ -62,14 +81,67 @@ Take a look to the example directory of the distribution if you need a code exam
 
 Call C<new()> to create a new log handler object.
 
-You can pass the options for the first log file if you want or later by the call of C<add()>.
+    my $log = Log::Handler->new();
 
 =head2 add()
 
-Call C<add()> to add a log file to the object. C<add()> expects the options for the log file.
-Each option will be set to a default value if not set.
+Call C<add()> to add a new logger.
 
-=head2 Log levels
+There are different ways to add a new logger object to the handler. In the following
+example I show you one simple way:
+
+    use Log::Handler;
+    use Log::Handler::Logger::File;
+
+    # the logger options (how to handle the file)
+    my %logger_options = (
+        timeformat      => '%Y/%m/%d %H:%M:%S',
+        newline         => 1,
+        prefix          => '%T [%L] %S: ',
+        postfix         => '',
+        maxlevel        => 'debug',
+        minlevel        => 'emergency',
+        die_on_errors   => 1,
+        trace           => 1,
+        debug           => 0,
+        debug_mode      => 2,
+        debug_skip      => 0,
+    );
+
+    # the options for the file
+    my %file_options = (
+        filename        => 'file.log',
+        filelock        => 1,
+        fileopen        => 1,
+        reopen          => 1,
+        mode            => 'append',
+        autoflush       => 1,
+        permissions     => '0660',
+        utf8            => 1,
+    );
+
+    # we creating the objects
+    my $log  = Log::Handler->new();
+    my $file = Log::Handler::Logger::File->new( \%file_options );
+
+    # now we add the file object to the handler and the logger options
+    $log->add( $file => \%logger_options );
+
+But it can be simplier! You can pass all options for the logger and for the
+file in one step. You just need to tell the handler what do you want to add.
+
+    use Log::Handler;
+
+    my %all_options = (%logger_options, %file_options);
+
+    $log->add( file => \%all_options );
+
+Further examples:
+
+    $log->add( email => \%all_options );
+    $log->add( forward => \%all_options );
+
+=head2 Log level methods
 
 There are eigth log level methods (thirteen with shortcuts):
 
@@ -111,7 +183,7 @@ Or maybe:
 
     $log->info("Hello World!", "How are you?");
 
-Both calls write to the log file (provided that the log level INFO would log)
+Both calls would log
 
     Feb 01 12:56:31 [INFO] Hello World! How are you?
 
@@ -138,26 +210,35 @@ Both calls write to the log file (provided that the log level INFO would log)
 =back
 
 These thirteen methods could be very useful if you want to kwow if the current log level
-would write the message to the log file. All methods returns TRUE if the handler would
-log it and FALSE if not. Example:
+would output the message. All methods returns TRUE if the current set of minlevel and
+maxlevel would log the message and FALSE if not. Example:
 
     $log->debug(Dumper(\%hash));
 
-This example would dump the hash in any case and handoff it to the log handler, 
-but this isn't that what we really want because it could costs a lot of resources.
+This example would dump the hash in any case and pass it to the log handler, 
+but that is not that what we really want!
 
-    $log->debug(Dumper(\%hash))
-       if $log->is_debug();
+    if ( $log->is_debug ) {
+        $log->debug(Dumper(\%hash));
+    }
 
 Now we dump the hash only if the current log level would log it.
 
 The methods C<is_note()>, C<is_warn()>, C<is_err()>, C<is_crit()> and C<is_emerg()>
 are just shortcuts.
 
+=head2 fatal(), is_fatal()
+
+This are special methods that can be used for CRITICAL, ALERT and EMERGENCY messages.
+A lot of people like to use just the levels DEBUG, INFO, WARN, ERROR and FATAL so I 
+though to implement this. You just have to set C<minlevel> to C<critical>, C<alert>
+or C<emergency> to use it.
+
 =head2 trace()
 
 This method is a special log level and very useful if you want to print C<caller()>
 informations to all log files.
+
 In contrast to the log level methods this method prints C<caller()> informations
 to all log files in any case and you don't need to activate the debugger with the
 option C<debug>. Example:
@@ -179,9 +260,20 @@ for more informations.
 
 =head2 errstr()
 
-Call C<errstr()> if you want to get the last error message. That is useful with
-C<die_on_errors>. If you set this option to C<0> then the handler wouldn't croak
-if a simple write operation fails. Set C<die_on_errors> to control it yourself.
+Call C<errstr()> if you want to get the last error message. This is useful with
+C<die_on_errors>. If you set C<die_on_errors> to C<0> then the handler wouldn't
+croak if a simple write operation fails. Set C<die_on_errors> to control it yourself.
+
+    use Log::Handler;
+
+    my $log = Log::Handler->new();
+
+    $log->add(file => {
+        filename      => 'file.log',
+        maxlevel      => 'info',
+        mode          => 'append',
+        die_on_errors => 0,
+    });
 
     $log->info("Hello World!") or die $log->errstr;
 
@@ -192,19 +284,7 @@ Or
     }
 
 The exception is that the handler croaks in any case if the call of C<new()> or C<add()>
-fails because on missing params or wrong settings!
-
-    my $log = Log::Handler->new(filename => 'file.log', mode => 'foo bar');
-
-That would croak, because the option C<mode> except C<append> or C<trunc> or C<excl>.
-
-If you set the option C<fileopen> to 1 - the default - to open the log file permanent
-and the call of C<new> fails then you can absorb the error.
-
-    my $log = Log::Handler->new(
-        filename        => 'file.log',
-        die_on_errors   => 0
-    ) or warn Log::Handler->errstr;
+fails because on missing or wrong settings!
 
 =head2 config()
 
@@ -214,127 +294,7 @@ With this method it's possible to load your logger configuration from a file.
 
 Take a look into the documentation of L<Log::Handler::Config> for more informations.
 
-=head1 OPTIONS
-
-=head2 filename
-
-With C<filename> you can set a file name, a GLOBREF or you can set a string as an alias
-for STDOUT or STDERR. The default is STDOUT for this option.
-
-Set a file name:
-
-    my $log = Log::Handler->new( filename => 'file.log'  );
-
-Set a GLOBREF
-
-    open FH, '>', 'file.log' or die $!;
-    my $log = Log::Handler->new( filename => \*FH );
-
-Or the same with
-
-    open my $fh, '>', 'file.log' or die $!;
-    my $log = Log::Handler->new( filename => $fh );
-
-Set STDOUT or STDERR
-
-    my $log = Log::Handler->new( filename => \*STDOUT );
-    # or
-    my $log = Log::Handler->new( filename => \*STDERR );
-
-If the option C<filename> is set in a config file and you want to debug to your screen then
-you can set C<*STDOUT> or C<*STDERR> as a string.
-
-    my $log = Log::Handler->new( filename => '*STDOUT' );
-    # or
-    my $log = Log::Handler->new( filename => '*STDERR' );
-
-That is not possible:
-
-    my $log = Log::Handler->new( filename => '*FH' );
-
-Note that if you set a GLOBREF to C<filename> some options will be forced (overwritten)
-and you have to control the handles yourself. The forced options are
-
-    fileopen => 1
-    filelock => 0
-    reopen   => 0
-
-=head2 filelock
-
-Maybe it's desirable to lock the log file by each write operation because a lot of processes
-write at the same time to the log file. You can set the option C<filelock> to 0 or 1.
-
-    0 - no file lock
-    1 - exclusive lock (LOCK_EX) and unlock (LOCK_UN) by each write operation (default)
-
-=head2 fileopen
-
-Open a log file transient or permanent.
-
-    0 - open and close the logfile by each write operation
-    1 - open the logfile if C<new()> called and try to reopen the
-        file if C<reopen> is set to 1 and the inode of the file has changed (default)
-
-=head2 reopen
-
-This option works only if option C<fileopen> is set to 1.
-
-    0 - deactivated
-    1 - try to reopen the log file if the inode changed (default)
-
-=head2 fileopen and reopen
-
-Please note that it's better to set C<reopen> and C<fileopen> to 0 on Windows
-because Windows unfortunately haven't the faintest idea of inodes.
-
-To write your code independent you should control it:
-
-    my $os_is_win = $^O =~ /win/i ? 0 : 1;
-
-    my $log = Log::Handler->new(
-       filename => 'file.log',
-       mode     => 'append',
-       fileopen => $os_is_win
-    );
-
-If you set C<fileopen> to 0 then it implies that C<reopen> has no importance.
-
-=head2 mode
-
-There are three possible modes to open a log file.
-
-    append - O_WRONLY | O_APPEND | O_CREAT
-    excl   - O_WRONLY | O_EXCL   | O_CREAT (default)
-    trunc  - O_WRONLY | O_TRUNC  | O_CREAT
-
-C<append> would open the log file in any case and appends the messages at
-the end of the log file.
-
-C<excl> would fail by open the log file if the log file already exists.
-This is the default option because some security reasons.
-
-C<trunc> would truncate the complete log file if it exists. Please take care
-to use this option.
-
-Take a look to the documentation of C<sysopen()> to get more informations.
-
-=head2 autoflush
-
-    0 - autoflush off
-    1 - autoflush on (default)
-
-=head2 permissions
-
-The option C<permissions> sets the permission of the file if it creates and must
-be set as a octal value. The permission need to be in octal and are modified
-by your process's current "umask".
-
-That means that you have to use the unix style permissions such as C<chmod>.
-C<0640> is the default permission for this option. That means that the owner got
-read and write permissions and users in the same group got only read permissions.
-All other users got no access.
-
-Take a look to the documentation of C<sysopen()> to get more informations.
+=head1 LOGGER OPTIONS
 
 =head2 timeformat
 
@@ -354,7 +314,7 @@ This helpful option appends a newline to the log message if it not exist.
     0 - inactive (default)
     1 - active - appends a newline to the log message if not exist
 
-=head2 PLACEHOLDER prefix, postfix
+=head2 prefix, postfix
 
 It's possible to define a C<prefix> and C<postfix> for each message that will
 be logged to the log file. The C<prefix> will be written before and the C<postfix>
@@ -369,8 +329,10 @@ The available placeholders are:
     %P  PID
     %H  Hostname
     %N  Newline
-    %C  Filename and line number where the logger was called
-    %S  The program name
+    %C  Caller - filename and line number where the logger was called
+    %S  Script - the program name
+    %M  Measurement - replaced with the time since the last call of the handler
+    %m  The same as %M just 3 internal decimal places (0.001)
 
 The default C<prefix> is set to "%T [%L] ". The C<postfix> is not defined by default.
 As example the following code
@@ -416,15 +378,10 @@ It's possible to set the log level as a string or as number. The default
 C<maxlevel> is 4 and the default C<minlevel> is 0.
 
 Example: If C<maxlevel> is set to 4 and C<minlevel> to 0 then the levels emergency (emerg),
-alert, critical (crit) and error (err) are active and would be logged to the log file.
+alert, critical (crit) and error (err) are active and would be logged.
 
 You can set both to 8 or C<nothing> if you don't want to log any message.
 
-=head2 rewrite_to_stderr
-
-Set this option to 1 if you want that Log::Handler prints messages to STDERR if the message
-couldn't print to the log file. The default is 0.
-
 =head2 die_on_errors
 
 Set C<die_on_errors> to 0 if you don't want that the handler croaks if normal operations fail.
@@ -435,20 +392,16 @@ Set C<die_on_errors> to 0 if you don't want that the handler croaks if normal op
 The exception is that the handler croaks in any case if the call of C<new()> fails because
 on missing params or wrong settings.
 
-=head2 utf8
-
-If this option is set to 1 then UTF-8 will be set with C<binmode()> on the output filehandle.
-
 =head2 trace
 
-With this options it's possible to disable the tracing for a log file. By default this
+With this options it's possible to disable the tracing for a logger. By default this
 option is set to 1 and tracing is enabled.
 
 =head2 debug
 
 You can activate a simple debugger that writes C<caller()> informations for each log level
-that would log to the log file. The debugger is logging all defined values except C<hints>
-and C<bitmask>. Set C<debug> to 1 to activate the debugger. The debugger is set to 0 by default.
+that would logged. The debugger is logging all defined values except C<hints> and C<bitmask>.
+Set C<debug> to 1 to activate the debugger. The debugger is set to 0 by default.
 
 =head2 debug_mode
 
@@ -460,11 +413,14 @@ The block mode looks like this:
     use warnings;
     use Log::Handler;
 
-    my $log = Log::Handler->new(
-       maxlevel   => 'debug',
-       debug      => 1,
-       debug_mode => 1
-    );
+    my $log = Log::Handler->new()
+
+    $log->add(file => {
+        filename   => '*STDOUT',
+        maxlevel   => 'debug',
+        debug      => 1,
+        debug_mode => 1
+    });
 
     sub test1 { $log->debug() }
     sub test2 { &test1; }
@@ -536,13 +492,15 @@ This option let skip the C<caller()> informations the count of C<debug_skip>.
 
     use Log::Handler;
 
-    my $log = Log::Handler->new(
+    my $log = Log::Handler->new();
+
+    $log->add(file => {
        filename => 'file1.log',
        mode     => 'append',
        newline  => 1,
        maxlevel => 7,
        minlevel => 0
-    );
+    });
 
     $log->debug("this is a debug message");
     $log->info("this is a info message");
@@ -581,26 +539,29 @@ Would log
     # create the log handler object
     my $log = Log::Handler->new;
 
-    $log->add(
-       filename => 'debug.log',
-       mode     => 'append',
-       maxlevel => 7,
-       minlevel => 7
-    );
+    $log->add(file => {
+        filename => 'debug.log',
+        mode     => 'append',
+        maxlevel => 7,
+        minlevel => 7,
+        trace    => 1,
+    });
 
-    $log->add(
-       filename => 'common.log',
-       mode     => 'append',
-       maxlevel => 6,
-       minlevel => 5
-    );
+    $log->add(file => {
+        filename => 'common.log',
+        mode     => 'append',
+        maxlevel => 6,
+        minlevel => 5,
+        trace    => 0,
+    });
 
-    $log->add(
-       filename => 'error.log',
-       mode     => 'append',
-       maxlevel => 4,
-       minlevel => 0
-    );
+    $log->add(file => {
+        filename => 'error.log',
+        mode     => 'append',
+        maxlevel => 4,
+        minlevel => 0,
+        trace    => 1,
+    });
 
     # log to debug.log
     $log->debug("this is a debug message");
@@ -621,21 +582,23 @@ Would log
     $log->emergency("this is a emergency message");
     $log->emerg("this is a emergency message as well");
 
-    # force caller() informations to all log files
+    # force caller() informations just to error.log and debug.log
     $log->trace("this message goes to all log files");
 
 =head2 Just a notice:
 
     use Log::Handler;
 
-    my $log = Log::Handler->new(
+    my $log = Log::Handler->new();
+
+    $log->add(file => {
        filename   => '/var/run/pid-file1',
        mode       => 'trunc',
        maxlevel   => 5,
        minlevel   => 5,
        prefix     => '',
        timeformat => ''
-    );
+    });
 
     $log->note($$);
 
@@ -644,20 +607,16 @@ Would truncate /var/run/pid-file1 and write just the pid to the logfile.
 =head2 Selfmade prefix:
 
     use Log::Handler;
-    use Sys::Hostname;
 
-    my $hostname =  hostname;
-    my $pid      =  $$;
-    my $progname =  $0;
-       $progname =~ s@.*[/\\]@@;
+    my $log = Log::Handler->new();
 
-    my $log = Log::Handler->new(
+    $log->add(file => {
        filename => "${progname}.log",
        mode     => 'append',
        maxlevel => 6,
        newline  => 1,
        prefix   => "%H[%P] [%L] %S: "
-    );
+    });
 
     $log->info("Hello World!");
     $log->warning("There is something wrong!");
@@ -672,11 +631,13 @@ Would log:
     use Log::Handler;
     use Data::Dumper;
 
-    my $log = Log::Handler->new(
+    my $log = Log::Handler->new();
+
+    $log->add(file => {
        filename   => 'file1.log',
        mode       => 'append',
        maxlevel   => 4,
-    );
+    });
 
     my %hash = (foo => 1, bar => 2);
 
@@ -685,31 +646,35 @@ Would log:
 
 Would NOT dump %hash to the $log object!
 
-=head2 die_on_errors example:
+=head1 EXTENSIONS
 
-    use Log::Handler;
-    use Data::Dumper;
+Do you want write further extensions? There are just some requirements!
 
-    my $log = Log::Handler->new(
-       filename      => 'file1.log',
-       mode          => 'append',
-       die_on_errors => 0
-    ) or die Log::Handler->errstr();
+Logger objects should provide a C<new()> and a C<write()> method.
 
-    if ($log->is_debug()) {
-       $log->debug("\n".Dumper(\%hash))
-          or die $log->errstr();
-    }
+Config plugins should provide a C<get_config()> routine.
+
+Take a look into the source or write me a mail if you have questions.
 
 =head1 PREREQUISITES
 
-    Fcntl             -  for flock(), O_APPEND, O_WRONLY, O_EXCL and O_CREATE
-    POSIX             -  to generate the time stamp with strftime()
-    Params::Validate  -  to validate all options
-    Devel::Backtrace  -  to backtrace caller()
-    Carp              -  to croak() on errors if die_on_errors is active
-    Sys::Hostname     -  to get the current hostname
-    Config::General   -  to load configuration from a file
+Prerequisites for all modules:
+
+    Carp
+    Config::General
+    Devel::Backtrace
+    Fcntl
+    Net::SMTP
+    Params::Validate
+    POSIX
+    Time::HiRes
+    Sys::Hostname
+    UNIVERSAL::require
+
+Just for the test suite:
+
+    File::Spec
+    Test::More
 
 =head1 EXPORTS
 
@@ -733,6 +698,11 @@ IRC: irc.perl.org#perl
 
 If you send me a mail then add Log::Handler into the subject.
 
+=head1 TODO
+
+    * Log::Handler::Logger::DBI
+    * Log::Handler::Logger::Socket
+
 =head1 COPYRIGHT
 
 Copyright (C) 2007 by Jonny Schulz. All rights reserved.
@@ -766,22 +736,26 @@ SUCH DAMAGES.
 =cut
 
 package Log::Handler;
-our $VERSION = '0.38_03';
 
 use strict;
 use warnings;
-use Log::Handler::Logger;
-use Log::Handler::Config;
+our $VERSION = '0.38_04';
+our $ERRSTR  = '';
 $Log::Handler::Logger::CALLER = 4;
-$Log::Handler::ERRSTR = '';
+
+use Carp;
+use Log::Handler::Config;
+use Log::Handler::Logger;
+use Log::Handler::Logger::File;
 
 # --------------------------------------------------------------------
 # The BEGIN block is used to generate the syslog methods and the is_*
 # methods. The syslog methods calling write() with the syslog level
 # as first uppercase argument if the current log level is valid.
 # The is_* methods are only used to check if the current set of max-
-# and minlevel would log the message to the log file and returns TRUE
-# or FALSE. The levels NOTE, ERR, CRIT and EMERG are just shortcuts.
+# and minlevel would log the message and returns TRUE or FALSE. The
+# levels NOTE, ERR, CRIT and EMERG are just shortcuts. Other levels
+# are special like FATAL, TRACE, etc.
 # --------------------------------------------------------------------
 
 BEGIN {
@@ -800,6 +774,16 @@ BEGIN {
         emergency => 'EMERGENCY',
         emerg     => 'EMERGENCY',
         trace     => 'TRACE',
+        fatal     => 'FATAL',
+
+# thinking about ......
+#        carp      => 'FATAL',
+#        croak     => 'FATAL',
+#        cluck     => 'FATAL',
+#        confess   => 'FATAL',
+#        die       => 'FATAL',
+#        exit      => 'NOTICE', # exit is normal, not fatal!
+
     );
 
     while ( my ($routine, $level) = each %LEVELS_BY_ROUTINE ) {
@@ -825,7 +809,7 @@ BEGIN {
                 return 1;
             };
 
-            next if $routine eq 'trace';
+            next if $level eq 'TRACE';
 
             # -------------------------------------------------------------
             # Creating the 8 is_ level methods - total 13 with shortcuts:
@@ -845,60 +829,26 @@ BEGIN {
 
 sub new {
     my $class = shift;
-    my $self  = bless { }, $class;
+    my $self  = bless { levels => { } }, $class;
     if (@_) {
-        $self->add(@_);
+        $self->add(File => @_);
     }
     return $self;
 }
 
 sub add {
-    my $self = shift;
+    @_ > 2 or Carp::croak 'Usage: $log->add($type => \%options)';
+    my $self   = shift;
     my $logger = Log::Handler::Logger->new(@_);
+    my $levels = $self->{levels};
 
-    # There are 2 ways to add the logger objects. The one way is to
-    # push all objects into a array or build a hash and if one of the
-    # log level methods is called the array or hash must be iterated
-    # to find out which logger would log. Example:
-    #
-    #       push @{$self->{logger}}, $logger;
-    #
-    #       sub info {
-    #           my $self = shift;
-    #           foreach my $logger ( @{$self->{logger}} ) {
-    #               if ($logger->would_log('INFO')) {
-    #                   $logger->write('INFO', @_) or return undef;
-    #               }
-    #           }
-    #           return 1;
-    #       }
-    #
-    # I though that's a bad way because if 4 log files are added then
-    # all logger objects would be checked by each call. For this reason
-    # I though it's better to build a HoA with all active levels as hashkeys
-    # and push the logger objects into the active levels. So it's doesn't
-    # necessary to iterate over all logger objects and it must be only
-    # checked if a level is defined and iterate over logger objects that
-    # are pushed into this array. Example:
-    #
-    #       push @{$self->{levels}->{INFO}}, $logger;
-    #
-    #       sub info {
-    #           my $self  = shift;
-    #           if ( $self->{levels}->{'INFO'} ) {
-    #               foreach my $logger ( @{$self->{levels}->{'INFO'}} ) {
-    #                   $logger->write('INFO', @_) or return undef;
-    #               }
-    #           }
-    #           return 1;
-    #       }
-    #
-    # That's much faster as the first example.
+    # Iterate all level and push the active levels into a HoA.
+    # This way makes it possible to check very fast if a log
+    # level is active and iterate over all objects.
 
-    # Iterate all level and if a level is active the logger is pushed into the array
-    foreach my $level (qw/DEBUG INFO NOTICE WARNING ERROR CRITICAL ALERT EMERGENCY TRACE/) {
+    foreach my $level (qw/DEBUG INFO NOTICE WARNING ERROR CRITICAL ALERT EMERGENCY FATAL TRACE/) {
         if ( $logger->would_log($level) ) {
-            push @{$self->{levels}->{$level}}, $logger;
+            push @{$levels->{$level}}, $logger;
         }
     }
 
@@ -908,9 +858,12 @@ sub add {
 sub config {
     my $self = shift;
     my $configs = Log::Handler::Config->config(@_);
-    $self->add($_) for @$configs;
+
+    while ( my ($type, $config) = each %$configs ) {
+        $self->add($type, $_) for @$config;
+    }
 }
 
-sub errstr { $Log::Handler::ERRSTR }
+sub errstr { $ERRSTR }
 
 1;
